' Variables for calculating shades of any RGB color value
Public Type ColorProperties
   red As Long
   green As Long
   blue As Long
   Hue As Double
   Sat As Double
   Lum As Double
End Type

Public Function HexToDec(HexCode As String) As Long
   Dim red As String
   Dim green As String
   Dim blue As String
   Dim NewHex As String
   NewHex = Right(HexCode, 6) 'strips the octothorp if there is one
   red = Left(NewHex, 2)
   green = Mid(NewHex, 3, 2)
   blue = Right(NewHex, 2)
   HexToDec = rgb("&H" & red, "&H" & green, "&H" & blue)
End Function

Public Function rgbRed(rgbColor As Long) As Long
    rgbRed = rgbColor Mod &H100
End Function

Public Function rgbGreen(rgbColor As Long) As Long
    rgbGreen = (rgbColor \ &H100) Mod &H100
End Function

Public Function rgbBlue(rgbColor As Long) As Long
    rgbBlue = (rgbColor \ &H10000) Mod &H100
End Function

Public Function RGBtoHSL(rgbColor As Long) As ColorProperties
   Dim r As Double, g As Double, b As Double
   Dim mn As Double, mx As Double
   Dim Delta As Double
   Dim DeltaR As Double, DeltaG As Double, DeltaB As Double
   Dim cp As ColorProperties

   'R, G and B input range = 0 to 255
   'H, S and L output range = 0 to 1.0

   r = (rgbRed(rgbColor) / 255)
   g = (rgbGreen(rgbColor) / 255)
   b = (rgbBlue(rgbColor) / 255)

   mn = Min(r, g, b)      ' Min. value of RGB
   mx = Max(r, g, b)      ' Max. value of RGB
   Delta = mx - mn        ' Delta RGB value

   'This is a gray, no chroma. Set the initial HSL values
   cp.Hue = 0
   cp.Sat = 0
   cp.Lum = (mx + mn) / 2      ' Lightness or Luminosity

   If Delta <> 0 Then
      ' Saturation
      If cp.Lum < 0.5 Then
         cp.Sat = Delta / (mx + mn)
      Else
         cp.Sat = Delta / (2 - mx - mn)
      End If

      ' Hue
      DeltaR = (((mx - r) / 6) + (Delta / 2)) / Delta
      DeltaG = (((mx - g) / 6) + (Delta / 2)) / Delta
      DeltaB = (((mx - b) / 6) + (Delta / 2)) / Delta

      If r = mx Then
         cp.Hue = DeltaB - DeltaG
      ElseIf g = mx Then
         cp.Hue = (1 / 3) + DeltaR - DeltaB
      Else
         cp.Hue = (2 / 3) + DeltaG - DeltaR
      End If
 
      ' Force between 0 and 1
      If cp.Hue < 0 Then cp.Hue = cp.Hue + 1
      If cp.Hue > 1 Then cp.Hue = cp.Hue - 1
      'debug.assert H >= 0 And H <= 1
   End If
   cp.Hue = cp.Hue * 240
   cp.Sat = cp.Sat * 240
   cp.Lum = cp.Lum * 240
   RGBtoHSL = cp

End Function

Public Function HSLtoRGB(cp As ColorProperties) As Long
   ' adapted from // http://www.easyrgb.com/
   Dim x1 As Double, x2 As Double, h As Double, s As Double, l As Double, _
      red As Double, green As Double, blue As Double

   h = cp.Hue / 360
   s = cp.Sat / 100
   l = cp.Lum / 100
  
   If s = 0 Then
      red = l * 255
      green = l * 255
      blue = l * 255
   Else
      If l < 0.5 Then
         x2 = l * (1 + s)
      Else
         x2 = (l + s) - (l * s)
      End If
      x1 = 2 * l - x2
      red = 255 * HUEtoRGB(x1, x2, h + (1 / 3))
      green = 255 * HUEtoRGB(x1, x2, h)
      blue = 255 * HUEtoRGB(x1, x2, h - (1 / 3))
 
   End If
   HSLtoRGB = rgb(red, green, blue)
 
End Function

Private Function HUEtoRGB(a As Double, b As Double, h As Double) As Double
   ' adapted from // http://www.easyrgb.com/
    If h < 0 Then h = h + 1
    If h > 1 Then h = h - 1
    Debug.Assert h >= 0 And h <= 1
 
    If (6 * h < 1) Then
        HUEtoRGB = a + (b - a) * 6 * h
    ElseIf (2 * h < 1) Then
        HUEtoRGB = b
    ElseIf (3 * h < 2) Then
        HUEtoRGB = a + (b - a) * ((2 / 3) - h) * 6
    Else
        HUEtoRGB = a
    End If
End Function

' Return Max RGB value
Public Function Max(ByVal r As Variant, ByVal g As Variant, ByVal b As Variant) As Variant
   If r > g And r > b Then
      Max = r
   ElseIf g > r And g > b Then
      Max = g
   Else
      Max = b
   End If
End Function

' Return Min RGB value
Public Function Min(ByVal r As Variant, ByVal g As Variant, ByVal b As Variant) As Variant
   If r < g And r < b Then
      Min = r
   ElseIf g < r And g < b Then
      Min = g
   Else
      Min = b
   End If
End Function